@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization
+  Copyright @ 1999 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.  You may
@@ -18,7 +18,6 @@
 #ifndef MAGICKCORE_PIXEL_ACCESSOR_H
 #define MAGICKCORE_PIXEL_ACCESSOR_H
 
-#include <assert.h>
 #include "MagickCore/cache.h"
 #include "MagickCore/cache-view.h"
 #include "MagickCore/color.h"
@@ -33,7 +32,7 @@ extern "C" {
 
 #undef index
 
-static inline Quantum ClampPixel(const MagickRealType pixel)
+static inline const Quantum ClampPixel(const MagickRealType pixel)
 {
   if (pixel < 0.0f)
     return((Quantum) 0);
@@ -46,13 +45,13 @@ static inline Quantum ClampPixel(const MagickRealType pixel)
 #endif
 }
 
-static inline Quantum GetPixela(const Image *magick_restrict image,
+static inline const Quantum GetPixela(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[aPixelChannel].offset]);
 }
 
-static inline Quantum GetPixelAlpha(const Image *magick_restrict image,
+static inline const Quantum GetPixelAlpha(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[AlphaPixelChannel].traits == UndefinedPixelTrait)
@@ -60,19 +59,19 @@ static inline Quantum GetPixelAlpha(const Image *magick_restrict image,
   return(pixel[image->channel_map[AlphaPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelAlphaTraits(
+static inline const PixelTrait GetPixelAlphaTraits(
   const Image *magick_restrict image)
 {
   return(image->channel_map[AlphaPixelChannel].traits);
 }
 
-static inline Quantum GetPixelb(const Image *magick_restrict image,
+static inline const Quantum GetPixelb(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[bPixelChannel].offset]);
 }
 
-static inline Quantum GetPixelBlack(const Image *magick_restrict image,
+static inline const Quantum GetPixelBlack(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[BlackPixelChannel].traits == UndefinedPixelTrait)
@@ -80,35 +79,37 @@ static inline Quantum GetPixelBlack(const Image *magick_restrict image,
   return(pixel[image->channel_map[BlackPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelBlackTraits(
+static inline const PixelTrait GetPixelBlackTraits(
   const Image *magick_restrict image)
 {
   return(image->channel_map[BlackPixelChannel].traits);
 }
 
-static inline Quantum GetPixelBlue(const Image *magick_restrict image,
+static inline const Quantum GetPixelBlue(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[BluePixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelBlueTraits(const Image *magick_restrict image)
+static inline const PixelTrait GetPixelBlueTraits(
+  const Image *magick_restrict image)
 {
   return(image->channel_map[BluePixelChannel].traits);
 }
 
-static inline Quantum GetPixelCb(const Image *magick_restrict image,
+static inline const Quantum GetPixelCb(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[CbPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelCbTraits(const Image *magick_restrict image)
+static inline const PixelTrait GetPixelCbTraits(
+  const Image *magick_restrict image)
 {
   return(image->channel_map[CbPixelChannel].traits);
 }
 
-static inline Quantum GetPixelChannel(const Image *magick_restrict image,
+static inline const Quantum GetPixelChannel(const Image *magick_restrict image,
   const PixelChannel channel,const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[channel].traits == UndefinedPixelTrait)
@@ -116,83 +117,86 @@ static inline Quantum GetPixelChannel(const Image *magick_restrict image,
   return(pixel[image->channel_map[channel].offset]);
 }
 
-static inline PixelChannel GetPixelChannelChannel(
+static inline const PixelChannel GetPixelChannelChannel(
   const Image *magick_restrict image,const ssize_t offset)
 {
   return(image->channel_map[offset].channel);
 }
 
-static inline ssize_t GetPixelChannelOffset(const Image *magick_restrict image,
-  const PixelChannel channel)
+static inline const ssize_t GetPixelChannelOffset(
+  const Image *magick_restrict image,const PixelChannel channel)
 {
   return(image->channel_map[channel].offset);
 }
 
-static inline PixelTrait GetPixelChannelTraits(
+static inline const PixelTrait GetPixelChannelTraits(
   const Image *magick_restrict image,const PixelChannel channel)
 {
   return(image->channel_map[channel].traits);
 }
 
-static inline size_t GetPixelChannels(const Image *magick_restrict image)
+static inline const size_t GetPixelChannels(const Image *magick_restrict image)
 {
   return(image->number_channels);
 }
 
-static inline Quantum GetPixelCompositeMask(const Image *magick_restrict image,
-  const Quantum *magick_restrict pixel)
+static inline const Quantum GetPixelCompositeMask(
+  const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[CompositeMaskPixelChannel].traits == UndefinedPixelTrait)
     return((Quantum) QuantumRange);
   return(pixel[image->channel_map[CompositeMaskPixelChannel].offset]);
 }
 
-static inline Quantum GetPixelCr(const Image *magick_restrict image,
+static inline const Quantum GetPixelCr(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[CrPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelCrTraits(const Image *magick_restrict image)
+static inline const PixelTrait GetPixelCrTraits(
+  const Image *magick_restrict image)
 {
   return(image->channel_map[CrPixelChannel].traits);
 }
 
-static inline Quantum GetPixelCyan(const Image *magick_restrict image,
+static inline const Quantum GetPixelCyan(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[CyanPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelCyanTraits(const Image *magick_restrict image)
+static inline const PixelTrait GetPixelCyanTraits(
+  const Image *magick_restrict image)
 {
   return(image->channel_map[CyanPixelChannel].traits);
 }
 
-static inline Quantum GetPixelGray(const Image *magick_restrict image,
+static inline const Quantum GetPixelGray(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[GrayPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelGrayTraits(const Image *magick_restrict image)
+static inline const PixelTrait GetPixelGrayTraits(
+  const Image *magick_restrict image)
 {
   return(image->channel_map[GrayPixelChannel].traits);
 }
 
-static inline Quantum GetPixelGreen(const Image *magick_restrict image,
+static inline const Quantum GetPixelGreen(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[GreenPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelGreenTraits(
+static inline const PixelTrait GetPixelGreenTraits(
   const Image *magick_restrict image)
 {
   return(image->channel_map[GreenPixelChannel].traits);
 }
 
-static inline Quantum GetPixelIndex(const Image *magick_restrict image,
+static inline const Quantum GetPixelIndex(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[IndexPixelChannel].traits == UndefinedPixelTrait)
@@ -200,13 +204,13 @@ static inline Quantum GetPixelIndex(const Image *magick_restrict image,
   return(pixel[image->channel_map[IndexPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelIndexTraits(
+static inline const PixelTrait GetPixelIndexTraits(
   const Image *magick_restrict image)
 {
   return(image->channel_map[IndexPixelChannel].traits);
 }
 
-static inline MagickRealType GetPixelInfoChannel(
+static inline const MagickRealType GetPixelInfoChannel(
   const PixelInfo *magick_restrict pixel_info,const PixelChannel channel)
 {
   switch (channel)
@@ -231,7 +235,7 @@ static inline MagickRealType GetPixelInfoChannel(
   }
 }
 
-static inline double PerceptibleReciprocal(const double x)
+static inline const double PerceptibleReciprocal(const double x)
 {
   double
     sign;
@@ -245,7 +249,7 @@ static inline double PerceptibleReciprocal(const double x)
   return(sign/MagickEpsilon);
 }
 
-static inline MagickRealType GetPixelInfoLuma(
+static inline const MagickRealType GetPixelInfoLuma(
   const PixelInfo *magick_restrict pixel)
 {
   MagickRealType
@@ -263,7 +267,7 @@ static inline MagickRealType GetPixelInfoLuma(
   return(intensity);
 }
 
-static inline MagickRealType GetPixelInfoLuminance(
+static inline const MagickRealType GetPixelInfoLuminance(
   const PixelInfo *magick_restrict pixel)
 {
   MagickRealType
@@ -281,32 +285,32 @@ static inline MagickRealType GetPixelInfoLuminance(
   return(intensity);
 }
 
-static inline Quantum GetPixelL(const Image *magick_restrict image,
+static inline const Quantum GetPixelL(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[LPixelChannel].offset]);
 }
 
-static inline ssize_t GetPixelLabel(const Image *magick_restrict image,
+static inline const ssize_t GetPixelLabel(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return((ssize_t) pixel[image->channel_map[LabelPixelChannel].offset]);
 }
 
-static inline MagickRealType GetPixelLuma(const Image *magick_restrict image,
-  const Quantum *magick_restrict pixel)
+static inline const MagickRealType GetPixelLuma(
+  const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   MagickRealType
     intensity;
 
-  intensity=(MagickRealType) (
-    0.212656f*pixel[image->channel_map[RedPixelChannel].offset]+
-    0.715158f*pixel[image->channel_map[GreenPixelChannel].offset]+
-    0.072186f*pixel[image->channel_map[BluePixelChannel].offset]);
+  intensity=
+    0.212656f*(MagickRealType) pixel[image->channel_map[RedPixelChannel].offset]+
+    0.715158f*(MagickRealType) pixel[image->channel_map[GreenPixelChannel].offset]+
+    0.072186f*(MagickRealType) pixel[image->channel_map[BluePixelChannel].offset];
   return(intensity);
 }
 
-static inline MagickRealType GetPixelLuminance(
+static inline const MagickRealType GetPixelLuminance(
   const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   MagickRealType
@@ -314,10 +318,10 @@ static inline MagickRealType GetPixelLuminance(
 
   if (image->colorspace != sRGBColorspace)
     {
-      intensity=(MagickRealType) (
-        0.212656f*pixel[image->channel_map[RedPixelChannel].offset]+
-        0.715158f*pixel[image->channel_map[GreenPixelChannel].offset]+
-        0.072186f*pixel[image->channel_map[BluePixelChannel].offset]);
+      intensity=
+        0.212656f*(MagickRealType) pixel[image->channel_map[RedPixelChannel].offset]+
+        0.715158f*(MagickRealType) pixel[image->channel_map[GreenPixelChannel].offset]+
+        0.072186f*(MagickRealType) pixel[image->channel_map[BluePixelChannel].offset];
       return(intensity);
     }
   intensity=(MagickRealType) (0.212656f*DecodePixelGamma((MagickRealType)
@@ -329,19 +333,33 @@ static inline MagickRealType GetPixelLuminance(
   return(intensity);
 }
 
-static inline Quantum GetPixelMagenta(const Image *magick_restrict image,
+static inline const Quantum GetPixelMagenta(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[MagentaPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelMagentaTraits(
+static inline const PixelTrait GetPixelMagentaTraits(
   const Image *magick_restrict image)
 {
   return(image->channel_map[MagentaPixelChannel].traits);
 }
 
-static inline Quantum GetPixelReadMask(const Image *magick_restrict image,
+static inline const Quantum GetPixelMeta(const Image *magick_restrict image,
+  const Quantum *magick_restrict pixel)
+{
+  if (image->channel_map[MetaPixelChannel].traits == UndefinedPixelTrait)
+    return(OpaqueAlpha);
+  return(pixel[image->channel_map[MetaPixelChannel].offset]);
+}
+
+static inline const PixelTrait GetPixelMetaTraits(
+  const Image *magick_restrict image)
+{
+  return(image->channel_map[MetaPixelChannel].traits);
+}
+
+static inline const Quantum GetPixelReadMask(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[ReadMaskPixelChannel].traits == UndefinedPixelTrait)
@@ -349,32 +367,33 @@ static inline Quantum GetPixelReadMask(const Image *magick_restrict image,
   return(pixel[image->channel_map[ReadMaskPixelChannel].offset]);
 }
 
-static inline Quantum GetPixelWriteMask(const Image *magick_restrict image,
-  const Quantum *magick_restrict pixel)
+static inline const Quantum GetPixelWriteMask(
+  const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[WriteMaskPixelChannel].traits == UndefinedPixelTrait)
     return((Quantum) QuantumRange);
   return(pixel[image->channel_map[WriteMaskPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelReadMaskTraits(
+static inline const PixelTrait GetPixelReadMaskTraits(
   const Image *magick_restrict image)
 {
   return(image->channel_map[ReadMaskPixelChannel].traits);
 }
 
-static inline size_t GetPixelMetaChannels(const Image *magick_restrict image)
+static inline const size_t GetPixelMetaChannels(
+  const Image *magick_restrict image)
 {
   return(image->number_meta_channels);
 }
 
-static inline size_t GetPixelMetacontentExtent(
+static inline const size_t GetPixelMetacontentExtent(
   const Image *magick_restrict image)
 {
   return(image->metacontent_extent);
 }
 
-static inline Quantum GetPixelOpacity(const Image *magick_restrict image,
+static inline const Quantum GetPixelOpacity(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   if (image->channel_map[AlphaPixelChannel].traits != BlendPixelTrait)
@@ -382,13 +401,14 @@ static inline Quantum GetPixelOpacity(const Image *magick_restrict image,
   return(QuantumRange-pixel[image->channel_map[AlphaPixelChannel].offset]);
 }
 
-static inline Quantum GetPixelRed(const Image *magick_restrict image,
+static inline const Quantum GetPixelRed(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[RedPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelRedTraits(const Image *magick_restrict image)
+static inline const PixelTrait GetPixelRedTraits(
+  const Image *magick_restrict image)
 {
   return(image->channel_map[RedPixelChannel].traits);
 }
@@ -421,53 +441,51 @@ static inline void GetPixelInfoPixel(const Image *magick_restrict image,
             pixel_info->black=(MagickRealType)
               pixel[image->channel_map[BlackPixelChannel].offset];
           if (image->channel_map[AlphaPixelChannel].traits != UndefinedPixelTrait)
-            {
-              pixel_info->alpha=(MagickRealType)
-                pixel[image->channel_map[AlphaPixelChannel].offset];
-              pixel_info->alpha_trait=BlendPixelTrait;
-            }
+            pixel_info->alpha=(MagickRealType)
+              pixel[image->channel_map[AlphaPixelChannel].offset];
           if (image->channel_map[IndexPixelChannel].traits != UndefinedPixelTrait)
             pixel_info->index=(MagickRealType)
-          pixel[image->channel_map[IndexPixelChannel].offset];
+              pixel[image->channel_map[IndexPixelChannel].offset];
         }
     }
 }
 
-static inline PixelTrait GetPixelTraits(const Image *magick_restrict image,
-  const PixelChannel channel)
+static inline const PixelTrait GetPixelTraits(
+  const Image *magick_restrict image,const PixelChannel channel)
 {
   return(image->channel_map[channel].traits);
 }
 
-static inline Quantum GetPixelY(const Image *magick_restrict image,
+static inline const Quantum GetPixelY(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[YPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelYTraits(const Image *magick_restrict image)
+static inline const PixelTrait GetPixelYTraits(
+  const Image *magick_restrict image)
 {
   return(image->channel_map[YPixelChannel].traits);
 }
 
-static inline Quantum GetPixelYellow(const Image *magick_restrict image,
+static inline const Quantum GetPixelYellow(const Image *magick_restrict image,
   const Quantum *magick_restrict pixel)
 {
   return(pixel[image->channel_map[YellowPixelChannel].offset]);
 }
 
-static inline PixelTrait GetPixelYellowTraits(
+static inline const PixelTrait GetPixelYellowTraits(
   const Image *magick_restrict image)
 {
   return(image->channel_map[YellowPixelChannel].traits);
 }
 
-static inline MagickRealType AbsolutePixelValue(const MagickRealType x)
+static inline const MagickRealType AbsolutePixelValue(const MagickRealType x)
 {
   return(x < 0.0f ? -x : x);
 }
 
-static inline MagickBooleanType IsPixelAtDepth(const Quantum pixel,
+static inline const MagickBooleanType IsPixelAtDepth(const Quantum pixel,
   const QuantumAny range)
 {
   Quantum
@@ -480,12 +498,12 @@ static inline MagickBooleanType IsPixelAtDepth(const Quantum pixel,
     (((MagickRealType) range*pixel)/QuantumRange+0.5)))/range+0.5);
 #else
   quantum=(Quantum) (((MagickRealType) QuantumRange*((QuantumAny)
-    (((MagickRealType) range*pixel)/QuantumRange+0.5)))/range);
+    (((MagickRealType) range*pixel)/QuantumRange+0.5)))/(MagickRealType) range);
 #endif
   return(pixel == quantum ? MagickTrue : MagickFalse);
 }
 
-static inline MagickBooleanType IsPixelEquivalent(
+static inline const MagickBooleanType IsPixelEquivalent(
   const Image *magick_restrict image,const Quantum *magick_restrict p,
   const PixelInfo *magick_restrict q)
 {
@@ -522,8 +540,8 @@ static inline MagickBooleanType IsPixelEquivalent(
   return(MagickTrue);
 }
 
-static inline MagickBooleanType IsPixelGray(const Image *magick_restrict image,
-  const Quantum *magick_restrict pixel)
+static inline const MagickBooleanType IsPixelGray(
+  const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   MagickRealType
     green_blue,
@@ -540,7 +558,7 @@ static inline MagickBooleanType IsPixelGray(const Image *magick_restrict image,
   return(MagickFalse);
 }
 
-static inline MagickBooleanType IsPixelInfoEquivalent(
+static inline const MagickBooleanType IsPixelInfoEquivalent(
   const PixelInfo *magick_restrict p,const PixelInfo *magick_restrict q)
 {
   MagickRealType
@@ -570,7 +588,7 @@ static inline MagickBooleanType IsPixelInfoEquivalent(
   return(MagickTrue);
 }
 
-static inline MagickBooleanType IsPixelMonochrome(
+static inline const MagickBooleanType IsPixelMonochrome(
   const Image *magick_restrict image,const Quantum *magick_restrict pixel)
 {
   MagickRealType
@@ -593,7 +611,7 @@ static inline MagickBooleanType IsPixelMonochrome(
   return(MagickFalse);
 }
 
-static inline MagickBooleanType IsPixelInfoGray(
+static inline const MagickBooleanType IsPixelInfoGray(
   const PixelInfo *magick_restrict pixel)
 {
   if ((AbsolutePixelValue(pixel->red-pixel->green) < MagickEpsilon) &&
@@ -602,7 +620,7 @@ static inline MagickBooleanType IsPixelInfoGray(
   return(MagickFalse);
 }
 
-static inline MagickBooleanType IsPixelInfoMonochrome(
+static inline const MagickBooleanType IsPixelInfoMonochrome(
   const PixelInfo *magick_restrict pixel_info)
 {
   MagickRealType
@@ -715,8 +733,10 @@ static inline void SetPixelChannelAttributes(
   const Image *magick_restrict image,const PixelChannel channel,
   const PixelTrait traits,const ssize_t offset)
 {
-  assert((ssize_t) channel < MaxPixelChannels);
-  assert(offset < MaxPixelChannels);
+  if ((ssize_t) channel >= MaxPixelChannels)
+    return;
+  if (offset >= MaxPixelChannels)
+    return;
   image->channel_map[offset].channel=channel;
   image->channel_map[channel].offset=offset;
   image->channel_map[channel].traits=traits;
@@ -834,6 +854,17 @@ static inline void SetPixelMagentaTraits(Image *image,const PixelTrait traits)
   image->channel_map[MagentaPixelChannel].traits=traits;
 }
 
+static inline void SetPixelMeta(const Image *magick_restrict image,
+  const Quantum red,Quantum *magick_restrict pixel)
+{
+  pixel[image->channel_map[MetaPixelChannel].offset]=red;
+}
+
+static inline void SetPixelMetaTraits(Image *image,const PixelTrait traits)
+{
+  image->channel_map[MetaPixelChannel].traits=traits;
+}
+
 static inline void SetPixelReadMask(const Image *magick_restrict image,
   const Quantum mask,Quantum *magick_restrict pixel)
 {
