@@ -30,16 +30,17 @@ extern "C" {
 #define MagickLibVersion  0x710
 #define MagickLibVersionText  "7.1.0"
 #define MagickLibVersionNumber  10,0,0
-#define MagickLibAddendum  "-27"
+#define MagickLibAddendum  "-32"
 #define MagickLibInterface  10
 #define MagickLibMinInterface  10
 #define MagickPlatform  "aarch64"
 #define MagickppLibVersionText  "7.1.0"
 #define MagickppLibVersionNumber  5:0:0
-#define MagickppLibAddendum  "-27"
+#define MagickppLibAddendum  "-32"
 #define MagickppLibInterface  5
 #define MagickppLibMinInterface  5
-#define MagickReleaseDate  "2022-03-04"
+#define MagickGitRevision  "20006"
+#define MagickReleaseDate  "2022-04-30"
 #define MagickAuthoritativeLicense  \
   "https://imagemagick.org/script/license.php"
 #define MagickAuthoritativeURL  "https://imagemagick.org"
@@ -68,7 +69,7 @@ extern "C" {
 #define MagickVersion  \
   MagickPackageName " " MagickLibVersionText MagickLibAddendum " " \
   MagickQuantumDepth MagickHDRISupport " " MagickPlatform " " \
-  MagickReleaseDate " " MagickAuthoritativeURL
+  MagickGitRevision " " MagickAuthoritativeURL
 
 extern MagickExport char
   *GetMagickHomeURL(void);
